#
#  Copyright 1999, 2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc perform_scan { record_list main } {

	set element [ $main.top_block.element_name get ]

	if { $element == "" } {
		set t "No element name has been specified."

		tk_dialog .bad_element_dialog \
		"No Element Selected" $t error 0 "OK"

		return
	}

	set edge [ $main.top_block.edge_selection get ]

	if { $edge == "" } {
		set t \
	"An absorption edge has not been selected for element '$element'."

		tk_dialog .bad_edge_dialog \
		"No Edge Selected" $t error 0 "OK"

		return
	}

	set edge_energy [ $main.top_block.edge_energy get ]

	if { $edge_energy == "" } {
		set t \
	"The absorption edge energy has not yet been specified."

		tk_dialog .bad_edge_energy_dialog \
		"No Edge Energy Selected" $t error 0 "OK"

		return
	}

	set type [ $main.middle_block.omenu get ]

	if { $type == "" } {
		set t "No scan type has been selected."

		tk_dialog .bad_scan_type_dialog \
		"No Scan Type Selected" $t error 0 "OK"

		return
	}

	set resolution [ $main.middle_block.ores get ]

	if { $resolution == "" } {
		set t "No scan resolution has been selected."

		tk_dialog .bad_scan_resolution_dialog \
		"No Scan Resolution Selected" $t error 0 "OK"

		return
	}

	set directory [ $main.bottom_block.directory get ]

	if { $directory == "" } {
		set t "No output directory has been selected."

		tk_dialog .bad_directory_dialog \
		"No Directory Selected" $t error 0 "OK"

		return
	}

	set type [ string tolower $type ]

	set resolution [ string tolower $resolution ]

	set upedge [ string toupper $edge ]

	log_puts \
"Starting scan '$type', resolution '$resolution' for $element $upedge edge."

	if { $type == "absorption" } {

		perform_absorption_scan $record_list $element $edge \
					$edge_energy $resolution $directory

	} elseif { $type == "fluorescence" } {

		perform_fluorescence_scan $record_list $element $edge \
					$edge_energy $resolution $directory

	} else {
		set t "The scan type '$type' does not exist."

		tk_dialog .bad_scan_type_dialog \
		"Unrecognized Scan Type" $t error 0 "OK"

		return
	}
}

proc perform_absorption_scan { record_list element edge edge_energy \
					resolution directory } {

	set filename [ construct_scan_filename $directory $element $edge ]

	log_puts "Output file = '$filename'"

	# Compute the scan parameters.

	set edge_energy $IMCAGui::edge_array($element.$edge)

	switch -exact -- $resolution {
		standard { set scan_name "mx_abs_standard" }
		short    { set scan_name "mx_abs_short" }
		fine     { set scan_name "mx_abs_fine" }
		coarse   { set scan_name "mx_abs_coarse" }
		default  { tk_dialog .unexpected_resolution \
				"Unexpected Resolution" \
		"The scan resolution '$resolution' was not recognized." \
				error 0 "OK"; return }
	}
 
	if [ catch { $record_list get_record $scan_name } scan_record ] {
		set t "Scan record '$scan_name' does not exist.  "
		append t "IMCA GUI cannot perform the absorption scan "
		append t "without it."

		tk_dialog .scan_does_not_exist_dialog "Scan Record Missing" \
			$t error 0 "OK"

		return
	}

	$scan_record set_field "datafile_name" $filename

	perform_scan_main $record_list $edge_energy \
				$directory $scan_record $filename "absorption"
}

proc perform_fluorescence_scan { record_list element edge edge_energy \
					resolution directory } {

	set filename [ construct_scan_filename $directory $element $edge ]

	log_puts "Output file = '$filename'"

	# Compute the scan parameters.

	set edge_energy $IMCAGui::edge_array($element.$edge)

	switch -exact -- $resolution {
		standard { set scan_name "mx_fl_standard" }
		short    { set scan_name "mx_fl_short" }
		fine     { set scan_name "mx_fl_fine" }
		coarse   { set scan_name "mx_fl_coarse" }
		default  { tk_dialog .unexpected_resolution \
				"Unexpected Resolution" \
		"The scan resolution '$resolution' was not recognized." \
				error 0 "OK"; return }
	}
 
	if [ catch { $record_list get_record $scan_name } scan_record ] {
		set t "Scan record '$scan_name' does not exist.  "
		append t "IMCA GUI cannot perform the fluorescence scan "
		append t "without it."

		tk_dialog .scan_does_not_exist_dialog "Scan Record Missing" \
			$t error 0 "OK"

		return
	}

	$scan_record set_field "datafile_name" $filename

	perform_scan_main $record_list $edge_energy \
				$directory $scan_record $filename "fluorescence"
}

proc perform_scan_main { record_list edge_energy directory \
				scan_record filename scan_name } {

	#
	# Change the edge energy
	#

	set status \
	  [ catch { $record_list get_record "edge_energy" } edge_energy_record ]

	if { $status != 0 } {
		set t "Cannot find 'edge_energy' record."

		tk_dialog .edge_energy_error_dialog \
		"Error Setting 'edge_energy'" $t error 0 "OK"

		return
	}

	set status [ catch { $edge_energy_record send_variable $edge_energy } ]

	if { $status != 0 } {
		set t "Cannot set 'edge_energy' value."

		tk_dialog .edge_energy_error_dialog \
		"Error Setting 'edge_energy'" $t error 0 "OK"

		return
	}

	#
	# Create a scan control window.
	#

	set window .scan_control_window

	frame $window
	pack $window -before .message_log

	label $window.scan_label -text "Current scan: "

	label $window.scan_name -text $scan_name

	pack $window.scan_label $window.scan_name -side left

	frame $window.sep -width 20
	pack $window.sep -side left -expand yes

	button $window.abort_scan -background red -activebackground red \
		-text "Abort Scan" -command [ code abort_scan_cmd ]

	pack $window.abort_scan -side right

	set sep .scan_control_window_sep

	frame $sep -height 2 -borderwidth 1 -relief sunken
	pack $sep -expand yes -fill x -pady 5 -before .message_log

	update

	#
	# Run the scan
	#

	set status [ catch { $scan_record perform_scan } ]

#	if { $status != 0 } {
#		set t \
#	"The scan '[ $scan_record cget -name ]' was abnormally terminated."
#
#		tk_dialog .perform_scan_error_dialog \
#		"Scan Aborted" $t error 0 "OK"
#
#		return
#	}

	#
	# Update the Chooch datafile name
	#

	set chooch_filename_widget \
			.experiment_window.bottom.input.chooch_filename

	$chooch_filename_widget clear

	$chooch_filename_widget insert end $filename

	#
	# Get rid of the scan control window.
	#

	update

	pack forget $window $sep
	destroy $window
	destroy $sep
}

proc abort_scan_cmd {} {
	log_puts "Scan abort requested."

	set Mx::user_interrupt 1
}

proc construct_scan_filename { directory element edge } {

	set filename_stem "${element}_${edge}_scan"

	set pathstem [ file join $directory $filename_stem ]

	set pattern "${pathstem}.\[0-9\]\[0-9\]\[0-9\]"

	set glob_result [ glob -nocomplain -- $pattern ]

	set glob_result [ lsort $glob_result ]

	if { $glob_result == "" } {

		set filename "$pathstem.001"
	} else {
		set last_filename [ lindex $glob_result end ]

		set extension_raw [ file extension $last_filename ]

		if { [ scan $extension_raw {.%d} extension ] != 1 } {
			set t "Can't parse filename extension '$extension_raw'"

			tk_dialog .parse_filename_extension_dialog \
			"Can't Parse Extension" $t error 0 "OK"

			return
		}

		if { $extension == "999" } {

			set new_extension "000"
		} else {

			set new_extension [ expr ( $extension + 1 ) ]

			set new_extension [ format {%03d} $new_extension ]
		}

		set filename "${pathstem}.${new_extension}"
	}

	return $filename
}

