#
#  Copyright 1999, 2002 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_main_spectroscopy_window { toplevel record_list } {

	set main $toplevel.main

	frame $main
	pack $main -side top -expand yes -fill both

	#-----------------------------------------------------------

	set frame $main.top_block

	frame $frame
	pack $frame -side top

	set ptable_window .periodic_table_window

	button $frame.periodic_table \
		-text "Periodic Table\nof the\nElements..." -command \
		[ code create_periodic_table_window $ptable_window 1 \
		element_state element_handler 1 periodic_table_customize ]

	pack $frame.periodic_table -side left

	#-----

	frame $frame.sep1 -width 1 -borderwidth 1
	pack $frame.sep1 -side left -expand yes -fill y -padx 5

	#-----

	label $frame.element_label -text Element
	pack $frame.element_label -side left

	iwidgets::entryfield $frame.element_name -width 3
	pack $frame.element_name -side left

	$frame.element_name configure -command \
		[ code spec_element_select_cmd $frame ]

	iwidgets::radiobox $frame.edge_selection
	pack $frame.edge_selection -side left

	$frame.edge_selection add k  -text K
	$frame.edge_selection add l1 -text L1
	$frame.edge_selection add l2 -text L2
	$frame.edge_selection add l3 -text L3
	$frame.edge_selection add m1 -text M1

	$frame.edge_selection buttonconfigure k -state disabled
	$frame.edge_selection buttonconfigure l1 -state disabled
	$frame.edge_selection buttonconfigure l2 -state disabled
	$frame.edge_selection buttonconfigure l3 -state disabled
	$frame.edge_selection buttonconfigure m1 -state disabled

	$frame.edge_selection configure -command \
		[ code spec_edge_select_cmd $frame ]

	label $frame.edge_label -text edge
	pack $frame.edge_label -side left

	#-----

	frame $frame.sep2 -width 1 -borderwidth 1
	pack $frame.sep2 -side left -expand yes -fill y -padx 5

	#-----

	iwidgets::spinner $frame.edge_energy -width 10
	pack $frame.edge_energy -side left

	$frame.edge_energy configure -validate \
					{ iwidgets::Entryfield::real %P }

	$frame.edge_energy configure -decrement \
			[ code edge_energy_decrement $frame.edge_energy ]

	$frame.edge_energy configure -increment \
			[ code edge_energy_increment $frame.edge_energy ]

	label $frame.edge_units -text " eV"
	pack $frame.edge_units -side left

	#-----------------------------------------------------------

	frame $main.horiz_sep0 -height 2 -borderwidth 1 -relief sunken
	pack $main.horiz_sep0 -side top -expand yes -fill x -pady 5

	set frame $main.scintillator_block

	frame $frame
	pack $frame -side top

	label $frame.scintillator_label -text \
				"Fluorescence Detector Saturation Test:"

	pack $frame.scintillator_label -side left

	frame $frame.sep1 -width 1 -borderwidth 1
	pack $frame.sep1 -side left -expand yes -fill y -padx 5

	button $frame.scintillator_test_start -text "Start Test" \
		-command [ code scintillator_test_start $record_list $frame ]

	pack $frame.scintillator_test_start -side left

	# Do not pack the stop test button until after the test starts.

	button $frame.scintillator_test_stop -text "Stop Test" \
		-bg red -activebackground red \
		-command [ code scintillator_test_stop $record_list $frame ]

	#-----------------------------------------------------------

	frame $main.horiz_sep1 -height 2 -borderwidth 1 -relief sunken
	pack $main.horiz_sep1 -side top -expand yes -fill x -pady 5

	set frame $main.middle_block

	frame $frame
	pack $frame -side top

	#-----

	iwidgets::optionmenu $frame.omenu -labeltext "Scan" \
		-command [ code scan_type_selected $frame ]

	pack $frame.omenu -side left

	$frame.omenu insert end Fluorescence Absorption 
	$frame.omenu select Fluorescence

	#-----

	frame $frame.sep1 -width 1 -borderwidth 1
	pack $frame.sep1 -side left -expand yes -fill y -padx 5

	#-----

	label $frame.directions -text "Mount Sample" \
					-font $IMCAGui::bold_italic_font

	pack $frame.directions -side left

	#-----

	frame $frame.sep2 -width 1 -borderwidth 1
	pack $frame.sep2 -side left -expand yes -fill y -padx 5

	#-----

	iwidgets::optionmenu $frame.ores \
		-command [ code scan_resolution_selected $frame ]

	pack $frame.ores -side left

	$frame.ores insert end Standard Short Fine Coarse
	$frame.ores select Standard

	#-----------------------------------------------------------

	frame $main.horiz_sep2 -height 2 -borderwidth 1
	pack $main.horiz_sep2 -side top -expand yes -fill x -pady 5

	set frame $main.bottom_block

	frame $frame
	pack $frame -side top

	#-----

	iwidgets::entryfield $frame.directory -labeltext "Output Directory"
	pack $frame.directory -side left

	$frame.directory clear
	$frame.directory insert 0 $IMCAGui::output_directory

	#-----

	button $frame.browse -text Browse -command \
		[ code select_directory $frame ]

	pack $frame.browse -side left

	#-----------------------------------------------------------

	frame $main.horiz_sep3 -height 2 -borderwidth 1
	pack $main.horiz_sep3 -side top -expand yes -fill x -pady 5

	button $main.go_button -text "Run Scan" \
		-command [ code perform_scan $record_list $main ]

	pack $main.go_button -side top

	#-----


}

proc edge_energy_increment { spinner } {

	if [ catch { $spinner get } edge_energy ] {

		tk_dialog .edge_energy_increment_dialog \
			"Increment button error" \
			"Error incrementing the edge energy" \
			error 0 "OK"

		return
	}

	set edge_energy [ expr ($edge_energy + 1.0) ]

	$spinner clear
	$spinner insert 0 $edge_energy
}

proc edge_energy_decrement { spinner } {

	if [ catch { $spinner get } edge_energy ] {

		tk_dialog .edge_energy_increment_dialog \
			"Decrement button error" \
			"Error decrementing the edge energy" \
			error 0 "OK"

		return
	}

	set edge_energy [ expr ($edge_energy - 1.0) ]

	$spinner clear
	$spinner insert 0 $edge_energy
}

proc select_directory { frame } {

	set dialog .directory_dialog

	if [ winfo exists $dialog ] {
		wm deiconify $dialog
		raise $dialog
		focus $dialog
		return
	}

	toplevel $dialog

	wm title $dialog "Select Output Directory"

	set directory [ $frame.directory get ]

	set selectionbox $dialog.selectionbox

	iwidgets::fileselectionbox $selectionbox
	pack $selectionbox

	$selectionbox configure -directory $directory
	$selectionbox configure -fileson 0
	$selectionbox configure -filetype directory
	$selectionbox configure -filteron 0

	set selection $selectionbox.selection

	$selection clear
	$selection insert 0 $directory

	set buttonbox $dialog.buttonbox

	iwidgets::buttonbox $buttonbox
	pack $buttonbox

	$buttonbox add OK -text OK -command \
			[ code select_directory_ok $dialog $frame ]

	$buttonbox add Cancel -text Cancel -command \
			[ code select_directory_cancel $dialog ]

	$buttonbox default OK

	wm deiconify $dialog
}

proc select_directory_ok { dialog frame } {

	set directory [ $dialog.selectionbox get ]

	set IMCAGui::output_directory $directory

	$frame.directory clear
	$frame.directory insert 0 $directory

	wm withdraw $dialog
}

proc select_directory_cancel { dialog } {

	wm withdraw $dialog
}

proc element_state { element } {

	if [ info exists IMCAGui::edge_array($element.k) ] {

		return "normal"
	}
	if [ info exists IMCAGui::edge_array($element.l1) ] {

		return "normal"
	}
	if [ info exists IMCAGui::edge_array($element.l2) ] {

		return "normal"
	}
	if [ info exists IMCAGui::edge_array($element.l3) ] {

		return "normal"
	}
	if [ info exists IMCAGui::edge_array($element.m1) ] {

		return "normal"
	}
	return "disabled"
}

proc element_handler { frame } {

	# If the menu entries have already been created, just return.

	if [ winfo exists $frame.button.tag ] {
		return
	}

	label $frame.button.tag			;# Do not pack or grid this.

	set element [ $frame.button cget -text ]

	set menu $frame.button.menu

	if [ info exists IMCAGui::edge_array($element.k) ] {
		set energy $IMCAGui::edge_array($element.k)

		$menu add command -label "K ($energy eV)" -command \
				[ code edge_select_handler_cmd $element k ]
	} else {
		$menu add command -label "K" -state disabled
	}

	if [ info exists IMCAGui::edge_array($element.l1) ] {
		set energy $IMCAGui::edge_array($element.l1)

		$menu add command -label "L1 ($energy eV)" -command \
				[ code edge_select_handler_cmd $element l1 ]
	} else {
		$menu add command -label "L1" -state disabled
	}

	if [ info exists IMCAGui::edge_array($element.l2) ] {
		set energy $IMCAGui::edge_array($element.l2)

		$menu add command -label "L2 ($energy eV)" -command \
				[ code edge_select_handler_cmd $element l2 ]
	} else {
		$menu add command -label "L2" -state disabled
	}

	if [ info exists IMCAGui::edge_array($element.l3) ] {
		set energy $IMCAGui::edge_array($element.l3)

		$menu add command -label "L3 ($energy eV)" -command \
				[ code edge_select_handler_cmd $element l3 ]
	} else {
		$menu add command -label "L3" -state disabled
	}

	if [ info exists IMCAGui::edge_array($element.m1) ] {
		set energy $IMCAGui::edge_array($element.m1)

		$menu add command -label "M1 ($energy eV)" -command \
				[ code edge_select_handler_cmd $element m1 ]
	} else {
		$menu add command -label "M1" -state disabled
	}
}

proc edge_select_handler_cmd { element edge } {

	set spec_frame .experiment_window.main.top_block

	set infoline .periodic_table_window.main_grid.infoline

	set energy $IMCAGui::edge_array($element.$edge)

	if [ winfo exists $infoline ] {
		$infoline.element_name configure -text $element
		$infoline.edge_name configure -text [ string toupper $edge ]
		$infoline.energy configure -text $energy
	}
	if [ winfo exists $spec_frame ] {
		$spec_frame.element_name clear
		$spec_frame.element_name insert end $element

		spec_element_select_cmd $spec_frame

		$spec_frame.edge_selection select $edge
	}
}

proc periodic_table_customize { window } {

	set spec_frame .experiment_window.main.top_block

	set infoline $window.main_grid.infoline

	frame $infoline
	grid $infoline -row 0 -column 3 -columnspan 9 -sticky w

	label $infoline.element_label -text "Selected Element:"
	pack  $infoline.element_label -side left

	label $infoline.element_name -text "--" -fg blue
	pack  $infoline.element_name -side left

	frame $infoline.spacer1 -width 10
	pack  $infoline.spacer1 -side left

	label $infoline.edge_name -text "--" -fg blue
	pack  $infoline.edge_name -side left

	label $infoline.edge_label -text edge
	pack  $infoline.edge_label -side left

	frame $infoline.spacer2 -width 10
	pack  $infoline.spacer2 -side left

	label $infoline.energy -text "---" -fg blue
	pack  $infoline.energy -side left

	label $infoline.energy_label -text eV
	pack  $infoline.energy_label -side left

	if [ winfo exists $spec_frame ] {
		set element [ $spec_frame.element_name get ]

		if { $element != "" } {
			$infoline.element_name configure -text $element
		}

		set edge [ $spec_frame.edge_selection get ]

		if { $edge != "" } {
			$infoline.edge_name configure -text \
				[ string toupper $edge ]
		}
		if [ info exists IMCAGui::edge_array($element.$edge) ] {

			$infoline.energy configure -text \
				$IMCAGui::edge_array($element.$edge)
		}
	}
}

proc spec_element_select_cmd { spec_frame } {

	set infoline .periodic_table_window.main_grid.infoline

	set element [ $spec_frame.element_name get ]

	set k_exists  [ info exists IMCAGui::edge_array($element.k) ]

	set l1_exists [ info exists IMCAGui::edge_array($element.l1) ]
	set l2_exists [ info exists IMCAGui::edge_array($element.l2) ]
	set l3_exists [ info exists IMCAGui::edge_array($element.l3) ]

	set m1_exists [ info exists IMCAGui::edge_array($element.m1) ]

	$spec_frame.edge_selection deselect k
	$spec_frame.edge_selection deselect l1
	$spec_frame.edge_selection deselect l2
	$spec_frame.edge_selection deselect l3
	$spec_frame.edge_selection deselect m1
	$spec_frame.edge_energy clear
	$spec_frame.edge_energy insert 0 ""

	if [ expr { ($k_exists == 0) \
		&& ($l1_exists == 0) \
		&& ($l2_exists == 0) \
		&& ($l3_exists == 0) \
		&& ($m1_exists == 0) \
	} ] {

		$spec_frame.edge_selection buttonconfigure k -state disabled
		$spec_frame.edge_selection buttonconfigure l1 -state disabled
		$spec_frame.edge_selection buttonconfigure l2 -state disabled
		$spec_frame.edge_selection buttonconfigure l3 -state disabled
		$spec_frame.edge_selection buttonconfigure m1 -state disabled

		if [ winfo exists $infoline ] {
			$infoline.element_name configure -text "--"
			$infoline.edge_name configure -text "--"
			$infoline.energy configure -text "---"
		}

		set t "The element '$element' is not available."

		tk_dialog .element_not_available_dialog \
		"Element Not Available" $t error 0 "OK"

		return
	}

	if { $k_exists } {
		$spec_frame.edge_selection buttonconfigure k -state normal
	} else {
		$spec_frame.edge_selection buttonconfigure k -state disabled
	}
	if { $l1_exists } {
		$spec_frame.edge_selection buttonconfigure l1 -state normal
	} else {
		$spec_frame.edge_selection buttonconfigure l1 -state disabled
	}
	if { $l2_exists } {
		$spec_frame.edge_selection buttonconfigure l2 -state normal
	} else {
		$spec_frame.edge_selection buttonconfigure l2 -state disabled
	}
	if { $l3_exists } {
		$spec_frame.edge_selection buttonconfigure l3 -state normal
	} else {
		$spec_frame.edge_selection buttonconfigure l3 -state disabled
	}
	if { $m1_exists } {
		$spec_frame.edge_selection buttonconfigure m1 -state normal
	} else {
		$spec_frame.edge_selection buttonconfigure m1 -state disabled
	}

	if [ winfo exists $infoline ] {
		$infoline.element_name configure -text $element
		$infoline.edge_name configure -text "--"
		$infoline.energy configure -text "---"
	}
}

proc spec_edge_select_cmd { spec_frame } {

	set infoline .periodic_table_window.main_grid.infoline

	set edge [ $spec_frame.edge_selection get ]

	if { [ string length $edge ] == 0 } {
		# We will arrive here when the GUI deselects a button.

		return
	}

	set element [ $spec_frame.element_name get ]

	set energy $IMCAGui::edge_array($element.$edge)

	$spec_frame.edge_energy clear
	$spec_frame.edge_energy insert 0 $energy

	if [ winfo exists $infoline ] {
		$infoline.element_name configure -text $element
		$infoline.edge_name configure -text [ string toupper $edge ]
		$infoline.energy configure -text $energy
	}
}

proc scintillator_test_start { record_list frame } {

	set scan_name "mx_saturation"

	if [ catch { $record_list get_record $scan_name } scan_record ] {
		set t "Scan record '$scan_name' does not exist.  "
		append t "IMCA GUI cannot perform the saturation test "
		append t "without it."

		tk_dialog .scan_does_not_exist_dialog "Scan Record Missing" \
			$t error 0 "OK"

		return
	}

	pack forget $frame.scintillator_test_start

	pack $frame.scintillator_test_stop -side left

	log_puts "Starting scan '$scan_name'."

	catch { $scan_record perform_scan }
}

proc scintillator_test_stop { record_list frame } {

	log_puts "Scan abort requested."

	set Mx::user_interrupt 1

	pack forget $frame.scintillator_test_stop

	pack $frame.scintillator_test_start -side left
}

proc scan_type_selected { frame } {

	set type [ $frame.omenu get ]

	switch -exact -- $type {
		Absorption { $frame.directions configure \
					-text "Insert Foil" \
					-font $IMCAGui::bold_italic_font }

		Fluorescence { $frame.directions configure \
					-text "Mount Sample" \
					-font $IMCAGui::bold_italic_font }
	}
}

proc scan_resolution_selected { frame } {

	#  Nothing here just now.

}
